package tessile;

import java.util.*;
import tessile.Materiale.TipoMateriale;
import tessile.MaterialeArredamento.TipoArredamento;

public class GestioneTessile {
	
	private Set<Materiale> materiali;
	private List<Fornitore> fornitori;
	private Map<Materiale,Fornitore> fornitorePerMateriale;
	
	public GestioneTessile(){
		this(new HashSet<Materiale>(),new ArrayList<Fornitore>(),new HashMap<Materiale,Fornitore>());
	}
	
	public GestioneTessile(Set<Materiale> materiali, List<Fornitore> fornitori, Map<Materiale,Fornitore> mappa){
		this.materiali = materiali;
		this.fornitori = fornitori;
		this.fornitorePerMateriale = mappa;
	}
	
	// metodo 1)
	public DatiMateriali contaMaterialiPerSettori(){
		int numAbbigliamento=0, numArredamento=0;
		Iterator<Materiale> i = materiali.iterator();
		while(i.hasNext()){
			Materiale m = i.next();
			if(m instanceof MaterialeAbbigliamento)
				numAbbigliamento++;
			else
				numArredamento++;
		}
		DatiMateriali dati = new DatiMateriali(numAbbigliamento,numArredamento);
		return dati;
	}
	
	/*
	// metodo 1) versione alternativa con for semplice
	public DatiMateriali contaMaterialiPerSettori(){
		int numAbbigliamento=0, numArredamento=0;
		for(Materiale m: materiali)
			if(m instanceof MaterialeAbbigliamento)
				numAbbigliamento++;
			else
				numArredamento++;
		DatiMateriali dati = new DatiMateriali(numAbbigliamento,numArredamento);
		return dati;
	}
	*/
	
	/*
	// metodo 1) versione alternativa con creazione e aggiornamento oggetto dati
	public DatiMateriali contaMaterialiPerSettori(){
		DatiMateriali dati = new DatiMateriali();
		Iterator<Materiale> i = materiali.iterator();
		while(i.hasNext()){
			Materiale m = i.next();
			if(m instanceof MaterialeAbbigliamento)
				dati.incrementaAbbigliamento();
			else
				dati.incrementaArredamento();
		}
		return dati;
	}
	*/
	
	// metodo 2)
	public void aggiornaFornitorePerMateriale(Fornitore fornitore, int codMateriale){
		boolean trovato = false;
		fornitori.add(fornitore);
		// cerco nella mappa il materiale corrispondente al codice dato
		Iterator<Materiale> i = fornitorePerMateriale.keySet().iterator();
		while(i.hasNext() && !trovato){
			Materiale materiale = i.next();
			if(materiale.getCodice() == codMateriale){
				trovato = true;
				fornitorePerMateriale.put(materiale,fornitore);
			}
		}	
	}
	
	// metodo 3)
	public boolean aggiungiMateriale(Materiale materiale){
		boolean esistente = false;
		
		if(materiali.contains(materiale))
			esistente = true;
		else // preferibile mettere else, anche se non richiesto
			materiali.add(materiale);
		return esistente;
	}
	
	/*
	// metodo 3) versione alternativa (notare che il testo richiede un booleano che indichi 
	// se il materiale inserito era gia' esistente. Il metodo add restituisce true se e' 
	// avvenuto l'inserimento, quindi se il materiale non era presente nel set. Per rispondere
	// alle esigenze del testo bisogna quindi anteporre l'operatore not (!))
	public boolean aggiungiMateriale(Materiale materiale){
		return !(materiali.add(materiale));
	}
	*/
	
	// metodo 4)
	public int fornitoriPerCitta(String citta){
		int numStampati=0;
		for(int i=0; i<fornitori.size();i++){
			Fornitore fornitore = fornitori.get(i);
			if(fornitore.getCitta().equals(citta)){
				System.out.println("Denominazione: " + fornitore.getDenominazione());
				System.out.println("Email: "+ fornitore.getEmail());
				System.out.println();
				numStampati++;
			}	
		}
		return numStampati;
	}
	

	// metodo 5)
	public int piuEconomico(String descrizione, TipoMateriale settore){
		int codiceFornitore = -1;
		double costoMinimo = Double.MAX_VALUE;
		Materiale economico = null;
		
		Iterator<Materiale> i = materiali.iterator();
		while(i.hasNext()){
			Materiale materiale = i.next();
			if(materiale.getDescrizione().equals(descrizione) && materiale.getTipoMateriale() == settore)
				if (materiale.getCostoAlMetro() < costoMinimo){
					economico = materiale;
					costoMinimo = economico.getCostoAlMetro();
				}
		}
		if(economico == null)
			System.out.println("Errore. Materiale con descrizione " + descrizione + " per il settore " + settore + " non trovato");
		else{
			Fornitore fornitore = fornitorePerMateriale.get(economico);
			codiceFornitore = fornitore.getCodice();
		}
		return(codiceFornitore);
	}
	
	/*
	// metodo 5) versione alternativa (pi pesante) con l'uso di instanceof
	public int piuEconomico(String descrizione, TipoMateriale settore){
		int codiceFornitore = -1;
		double costoMinimo = Double.MAX_VALUE;
		Materiale economico = null;
		
		Iterator<Materiale> i = materiali.iterator();
		while(i.hasNext()){
			Materiale materiale = i.next();
			if(materiale.getDescrizione().equals(descrizione))
				switch(settore){
				case ABBIGLIAMENTO:
					if(materiale instanceof MaterialeAbbigliamento)
						if (materiale.getCostoAlMetro() < costoMinimo){
							economico = materiale;
							costoMinimo = economico.getCostoAlMetro();
						}
					break;
				case ARREDAMENTO:
					if(materiale instanceof MaterialeArredamento)
						if (materiale.getCostoAlMetro() < costoMinimo){
							economico = materiale;
							costoMinimo = economico.getCostoAlMetro();
						}
					break;
				}
		}
		if(economico == null)
			System.out.println("Errore. Materiale con descrizione " + descrizione + " per il settore " + settore + " non trovato");
		else{
			Fornitore fornitore = fornitorePerMateriale.get(economico);
			codiceFornitore = fornitore.getCodice();
		}
		return(codiceFornitore);
	}
	*/
	
	// metodo 6) richiamo a metodo gia' definito
	public void fornitorePiuEconomico(String descrizione, TipoMateriale settore){
		boolean trovato = false;
		
		int codFornitore = piuEconomico(descrizione,settore);
		if(codFornitore != -1){
			Iterator<Fornitore> i = fornitori.iterator();
			while(i.hasNext() && !trovato){
				Fornitore fornitore = i.next();
				if(fornitore.getCodice() == codFornitore){
					trovato = true;
					System.out.println(fornitore); // chiamata implicita a toString()
				}
			}
		}
		else
			System.out.println("Errore: codice di fornitore non valido");
		
	}
	
	
	// metodo 7) uso di cast necessari
	public void statisticheMateriali(){
		int totAbbigliamento=0;
		int totArredamento=0;
		int totTendaggi=0;
		int totDivani=0;
		int totBiancheria=0;
		
		Iterator<Materiale> i = materiali.iterator();
		while(i.hasNext()){
			Materiale materiale = i.next();
			if(materiale instanceof MaterialeAbbigliamento)
				totAbbigliamento++;
			else{
				totArredamento++;
				// notare il cast obbligatorio perch getTipoArredamento  definito solo per MaterialeArredamento
				if(((MaterialeArredamento)materiale).getTipoArredamento() == TipoArredamento.TENDAGGI)
					totTendaggi++;
				else if(((MaterialeArredamento)materiale).getTipoArredamento() == TipoArredamento.DIVANI)
					totDivani++;
				else
					totBiancheria++;
			}
		}
		System.out.print("La percentuale di materiali per l'abbigliamento : ");
		System.out.println((double)totAbbigliamento/materiali.size()*100);
		System.out.print("La percentuale di materiali per l'arredamento : ");
		System.out.println((double)totArredamento/materiali.size()*100);
		System.out.print("La percentuale di materiali di arredamento per i tendaggi : ");
		System.out.println((double)totTendaggi/totArredamento*100);
		System.out.print("La percentuale di materiali di arredamento per i divani : ");
		System.out.println((double)totDivani/totArredamento*100);
		System.out.print("La percentuale di materiali di arredamento per la biancheria : ");
		System.out.println((double)totBiancheria/totArredamento*100);
	}
	
	
	/*
	// metodo 7) alternativo con uso di switch
	public void statisticheMateriali(){
		int totAbbigliamento=0;
		int totArredamento=0;
		int totTendaggi=0;
		int totDivani=0;
		int totBiancheria=0;
		
		Iterator<Materiale> i = materiali.iterator();
		while(i.hasNext()){
			Materiale materiale = i.next();
			if(materiale instanceof MaterialeAbbigliamento)
				totAbbigliamento++;
			else{
				totArredamento++;
				// notare il cast obbligatorio perch getTipoArredamento  definito solo per MaterialeArredamento
				TipoArredamento tipo = ((MaterialeArredamento)materiale).getTipoArredamento();
				switch(tipo){
				case TENDAGGI:
					totTendaggi++;
					break;
				case DIVANI:
					totDivani++;
					break;
				case BIANCHERIA:
					totBiancheria++;
					break;
				}	
			}
		}
		System.out.print("La percentuale di materiali per l'abbigliamento : ");
		System.out.println((double)totAbbigliamento/materiali.size()*100);
		System.out.print("La percentuale di materiali per l'arredamento : ");
		System.out.println((double)totArredamento/materiali.size()*100);
		System.out.print("La percentuale di materiali di arredamento per i tendaggi : ");
		System.out.println((double)totTendaggi/totArredamento*100);
		System.out.print("La percentuale di materiali di arredamento per i divani : ");
		System.out.println((double)totDivani/totArredamento*100);
		System.out.print("La percentuale di materiali di arredamento per la biancheria : ");
		System.out.println((double)totBiancheria/totArredamento*100);
	}
	*/
	
	// metodo 8) uso del metodo polimorfo costoFornitura
	public double costoFornituraPerMateriale(int codiceMateriale, int metri){
		boolean trovato = false;
		double costo=-1;
		
		Iterator<Materiale> i = materiali.iterator();
		while(i.hasNext() && !trovato){
			Materiale materiale = i.next();
			if(materiale.getCodice() == codiceMateriale){
				trovato = true;
				costo = materiale.costoFornitura(metri);
			}
		}
		return(costo);
	}
	
	// metodo 9) esempio di scorrimento di una mappa
	public void fornitoriPerMateriali(){
		Set<Materiale> dominio = fornitorePerMateriale.keySet();
		Iterator<Materiale> k = dominio.iterator();
		int i=0;
		while(k.hasNext()){
			i++;
			Materiale materiale = k.next();
			System.out.println("Materiale " + i + ": " + materiale);
			Fornitore fornitore = fornitorePerMateriale.get(materiale);
			System.out.println("Fornitore: " + fornitore);
		}
	}
}
